// ===================================================================
//
// BBDrive_Eye_Test.c
// Investigating eye response
//
// ==================================================================
   
#include <webots/robot.h>
#include <webots/motor.h> 
#include <stdio.h>
#include <webots/light_sensor.h>
#include <webots/gps.h>

#include "CBP_Helper_4.h"
#include "CBP_Plotter_4a.h"

#define TIME_STEP 32
#define WHEEL_RADIUS 0.0205
#define AXLE_LENGTH 0.053
#define MAX_SPEED 6.28
#define PI 3.14159265358979
 
WbDeviceTag left_motor, right_motor;
WbDeviceTag ls0, ls7;
WbDeviceTag gps;

// Neural stuff -------------------------
float tau = 0.01;
float inL1, inR1;
float outL1, outR1;
float outL2, outR2;
float A = 250.0;
float B = 100.0;
float offset = 500;
float C = 25;
float D = 7.5;

// ODE stuff ----------------
float u[4];
float dudt[4];
float deltaT = 0.01;

// Motor stuff --------------
float driveL, driveR;
float maxDrive = 50;

void deviceInit() {
 // get a handler to the motors and set target position to infinity (speed control).
  left_motor = wb_robot_get_device("left wheel motor");
  right_motor = wb_robot_get_device("right wheel motor");
  wb_motor_set_position(left_motor, INFINITY);
  wb_motor_set_position(right_motor, INFINITY);
  wb_motor_set_velocity(left_motor, 0.0);
  wb_motor_set_velocity(right_motor, 0.0);

  ls0 = wb_robot_get_device("ls0");
  wb_light_sensor_enable(ls0, TIME_STEP); 
  ls7 = wb_robot_get_device("ls7");
  wb_light_sensor_enable(ls7, TIME_STEP); 
  
  gps = wb_robot_get_device("gps");
  wb_gps_enable(gps, TIME_STEP);
       
  }  

int main(int argc, char **argv) {

  double time = 0;
  double locnX, locnY;
  
  FILE* plotFile;
  plotFile = fopen("BBDrive_Test_2.m", "w");
  writeOctaveHead(plotFile); 
  
  wb_robot_init();
 
  deviceInit();

  while (wb_robot_step(TIME_STEP) != -1) {
          
    double LDR_L = wb_light_sensor_get_value(ls7);    
    double LDR_R = wb_light_sensor_get_value(ls0);    
    
    // Emulate Arduino analogRead using ORP-12 LDRs
    inL1 = analogRead(LDR_L);
    inR1 = analogRead(LDR_R);

    // use GPS to get actual robot position
    const double *gpsVals = wb_gps_get_values(gps);
    locnX = gpsVals[0];
    locnY = gpsVals[1];
        
    driveL = 20;
    driveR = 20;
    
    // emulate Parallax Servos ----------------------
    double omegaL = (MAX_SPEED/50.0)*driveL;
    double omegaR = (MAX_SPEED/50.0)*driveR;  
    
    // Useful alt prints to check behaviour of processing steps
    // printf("inL1 = %f,  inR1 = %f\n",inL1,inR1);
    // printf("avL = %f, avR = %f\n",avL,avR);   
    // printf("inL1 = %f, inR1 = %f, u[0] = %f, u[1] = %f\n",inL1,inR1,u[0],u[1]);
    // printf("outL1 = %f, outR1 = %f\n",outL1,outR1);
    // printf("u[2] = %f, u[3] = %f\n",u[2],u[3]);
    // printf("inL1 = %f, inR1 = %f, u[0] = %f, u[1] = %f\n",inL1,inR1,u[0],u[1]);
    printf("locnX = %f, LDR_L = %f, LDR_R = %f, inL1 = %f, inR1 = %f\n",locnX,LDR_L,LDR_R,inL1,inR1);
     //printf("outL2 = %f, outR2 = %f, drivvL = %f, driveR = %f, omegaL = %f, omegaR = %f\n",
    //        outL2,outR2,driveL,driveR,omegaL,omegaR);
    
    time += (double)(TIME_STEP);
        
    writeDataToOctave(plotFile,time, LDR_L, LDR_R, inL1,inR1, locnX, locnY);
          
     
    wb_motor_set_velocity(left_motor,  omegaL);
    wb_motor_set_velocity(right_motor, omegaR);

  };

  writeOctaveTail(plotFile);
  fclose(plotFile);
  wb_robot_cleanup();

  return 0;
}
