// ===================================================================
// File:          CBP_ePuck_Odometry_1
// Date: 06-09-23
// 11-08-20 05-09-20 07-09-20 12-09-20 03-10-20 DEVEL
// 03-08-23
// Move along a striaght line to compare wheel encoders with GPS
// Produces an Octave file.
//
// ==================================================================

#include <webots/robot.h>
#include <webots/motor.h> 
#include <webots/distance_sensor.h>
#include <webots/position_sensor.h>
#include <webots/gps.h>

#include <stdio.h>

#include "CBP_Plotter_1.h"

#define TIME_STEP 32
#define WHEEL_RADIUS 0.0205
#define AXLE_LENGTH 0.053
#define MAX_SPEED 6.28
#define PI 3.14159265358979

#define STATE_MOVE 1
#define STATE_DONE 2

WbDeviceTag left_motor, right_motor, gps;

// This function initializes some components of the ePuck ---------
void init() {

  left_motor = wb_robot_get_device("left wheel motor");
  right_motor = wb_robot_get_device("right wheel motor");
  wb_motor_set_position(left_motor, INFINITY);
  wb_motor_set_position(right_motor, INFINITY);
  wb_motor_set_velocity(left_motor, 0.0);
  wb_motor_set_velocity(right_motor, 0.0);
   
  gps = wb_robot_get_device("gps");
  wb_gps_enable(gps, TIME_STEP); 

}


int main(int argc, char **argv) {

  double omegaL, omegaR;
  double desiredDistance;
  double desiredTime;
  double speed;
  double time = 0;
  int state = STATE_MOVE;
  WbDeviceTag leftWheelAngle;
    
  FILE* plotFile;
  plotFile = fopen("Robot.m", "w");
  writeOctaveHead(plotFile);  
  
  desiredDistance = 0.5;
  desiredTime = 7.12;
  speed = desiredDistance/desiredTime;
  omegaL = speed/WHEEL_RADIUS;
  omegaR = omegaL;
    
  printf("speed = %f, omegaL = %f, omegaR = %f\n",speed, omegaL, omegaR);
  
  wb_robot_init();

  init();
 
  // Enable the left wheel encoder ---------------------------------------
  leftWheelAngle = wb_robot_get_device("left wheel sensor");
  wb_position_sensor_enable(leftWheelAngle,TIME_STEP);
        
  // This loop runs indefinately until we stop the simulation ------------
  while (wb_robot_step(TIME_STEP) != -1) {
  
    switch(state) {
    
      case STATE_MOVE:
        printf("\n"); // To fool compiler
        const double *gpsVals = wb_gps_get_values(gps);   
        printf("GPS actual distance moved %f  ",gpsVals[0]); 
        
        // Get encoder value and use to calculate perceived distance moved.
        double wheelTheta = wb_position_sensor_get_value(leftWheelAngle);
        double dist = wheelTheta*WHEEL_RADIUS;
 
        printf("Perceived distance moved = %f ",dist);
        printf("Time = %f\n",time);
        writeData2ToOctave(plotFile,time,gpsVals[0],dist); 
        
        // Stop on distance travelled --------------------------------
        if(dist >= desiredDistance) {
          omegaL = 0.0;
          omegaR = 0.0;           
          state = STATE_DONE;    
        }         
             
        time += TIME_STEP/1000.0;        
        break;
        
      case STATE_DONE:
        writeOctaveTail(plotFile);
        fclose(plotFile);
        break;      
    
    }

    // Here we actually drive the motors with their angular velocities
    wb_motor_set_velocity(left_motor,  omegaL);
    wb_motor_set_velocity(right_motor, omegaR);

  };

  wb_robot_cleanup();

  return 0;
}
