// ===================================================================
// File:          CBP_ePuck_Odometry_10a
// Date:          30-08-23
// Status:        Devel
// orig 03-08-23
// Move along a striaght line to compare wheel encoders with GPS
// Simulates encoders
// Produces an Octave file.
// Should look like CBP_2403_D_Encoder_Test_2b
// ==================================================================

#include <webots/robot.h>
#include <webots/motor.h> 
#include <webots/distance_sensor.h>
#include <webots/position_sensor.h>
#include <webots/gps.h>

#include <stdio.h>

#include "CBP_Plotter_1.h"

#define TIME_STEP 32
#define WHEEL_RADIUS 0.0205
#define AXLE_LENGTH 0.053
#define MAX_SPEED 6.28
#define PI 3.14159265358979

#define STATE_MOVE 1
#define STATE_CALCULATE 2
#define STATE_DONE 3

#define FLOAT_TO_INT(x) ((x)>=0?(int)((x)+0.5):(int)((x)-0.5))

// Global variables ----------------------------------------------

double theta, dTheta;
WbDeviceTag left_motor, right_motor, gps;
WbDeviceTag leftWheelAngle;
// ---------------------------------------------------------------
void init() {

  left_motor = wb_robot_get_device("left wheel motor");
  right_motor = wb_robot_get_device("right wheel motor");
  wb_motor_set_position(left_motor, INFINITY);
  wb_motor_set_position(right_motor, INFINITY);
  wb_motor_set_velocity(left_motor, 0.0);
  wb_motor_set_velocity(right_motor, 0.0);
   
  gps = wb_robot_get_device("gps");
  wb_gps_enable(gps, TIME_STEP); 

  // get the left wheel encoder
  leftWheelAngle = wb_robot_get_device("left wheel sensor");
  wb_position_sensor_enable(leftWheelAngle,TIME_STEP);
}

// ----------------------------------------------------------------
// Simulates the Arduino encoder ISR
unsigned int ISR_Left() {

  unsigned int count;
  
  count = FLOAT_TO_INT((theta/dTheta));
  
  return count;
}

// ----------------------------------------------------------------
int main(int argc, char **argv) {

  double omegaL, omegaR;
  double desiredDistance;
  double desiredTime;
  double speed;
  double time = 0;  
  double dx;
  unsigned int nL;
  unsigned int count = 0;
  double percDist;
    
  int state = STATE_MOVE;

  FILE* plotFile;
  plotFile = fopen("Robot.m", "w");
  writeOctaveHead(plotFile);  
  
  dx = (2*PI*WHEEL_RADIUS)/1000;
  dTheta = 2*PI/1000;
  
  // Set the speed low enough so the encoder simulation works.
  // Can see this by looking at the count when the robot stops
  desiredDistance = 0.25;
  desiredTime = 25.0;
  
  // Set the wheel angular velocities --------------------------
  // Slightly different from Arduino where we chose driveL and
  // driveR arbitrarily since we had no real idea of actual
  // speeds. Here we do, so we can give the omegas sensible
  //values
  speed = desiredDistance/desiredTime;
  omegaL = speed/WHEEL_RADIUS;
  omegaR = omegaL;
  printf("Orig omegaL = omegaR = %f\n",omegaL);
  
  // Calc number of steps needed -------------------------------
  nL = (int)(desiredDistance / dx);
  
  /*
  // Correct the counts and speeds -----------------------------
  // correct nL
  nL = (int)((float)nL*0.249882/0.242587);
  // correct speeds
  omegaL = omegaL*0.249882/0.242587;
  omegaR = omegaL;
  printf("Corrected nL = nR = %d\n",nL);
  printf("Corr omegaL = omegaR = %f\n",omegaL);
   */
   
  wb_robot_init();

  init();
   
  while (wb_robot_step(TIME_STEP) != -1) {
  
    switch(state) {
    
      case STATE_MOVE:
        printf("\n"); // To fool compiler
        
        // use GPS to get actual robot position
        const double *gpsVals = wb_gps_get_values(gps);   
        
        // Get current wheel angle from the ePuck
        theta  = wb_position_sensor_get_value(leftWheelAngle);

        count = ISR_Left();
        percDist = count*dx;
        
        printf("Count = %d, Perc dist = %f, actual dist = %f\n",
                 count, percDist, gpsVals[0]);

        writeData2ToOctave(plotFile,time,gpsVals[0],percDist); 
                
        if(count >= nL){
          omegaL = 0;
          omegaR = 0;
          state = STATE_DONE;
        }
        
        time += TIME_STEP/1000.0;        
        break;
        
      case STATE_DONE:
        writeOctaveTail(plotFile);
        fclose(plotFile);
        break;      
    
    }
   
    wb_motor_set_velocity(left_motor,  omegaL);
    wb_motor_set_velocity(right_motor, omegaR);

  };

  wb_robot_cleanup();

  return 0;
}
